(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('that-box2d-library'), require('@angular/core'), require('ngx-component-mixins'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define(['exports', 'that-box2d-library', '@angular/core', 'ngx-component-mixins', '@angular/common'], factory) :
    (factory((global.ngxbox2d = {}),null,global.ngxbox2d,null,null));
}(this, (function (exports,thatBox2dLibrary,core,ngxComponentMixins,common) { 'use strict';

    /**
     * A utility class for performing SVG maths
     */
    var SVGMathsUtility = {
        /**
         * Convert a Box2D fixture into an SVG path
         */
        fixturePath: function (body, fixture) {
            var path = '', shape = fixture.m_shape;
            // If the Shape is a Polygon
            if (shape instanceof thatBox2dLibrary.B2PolygonShape) {
                var polyShape_1 = shape;
                polyShape_1.m_vertices.forEach(function (v, i) {
                    if (i < polyShape_1.m_count) {
                        var vw = body.GetWorldPoint(v, new thatBox2dLibrary.B2Vec2(0, 0));
                        path += '' + (i === 0 ? 'M ' : 'L ') + vw.x + ' ' + vw.y + ' ';
                    }
                });
                path +=
                    'L ' +
                        body.GetWorldPoint(polyShape_1.m_vertices[0], new thatBox2dLibrary.B2Vec2(0, 0)).x +
                        ' ' +
                        body.GetWorldPoint(polyShape_1.m_vertices[0], new thatBox2dLibrary.B2Vec2(0, 0)).y;
            }
            // Done
            return path;
        }
    };

    var __extends = (undefined && undefined.__extends) || (function () {
        var extendStatics = function (d, b) {
            extendStatics = Object.setPrototypeOf ||
                ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
                function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
            return extendStatics(d, b);
        };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __metadata = (undefined && undefined.__metadata) || function (k, v) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
    };
    /**
     * A component for attaching a Box2D body to a Angular Component
     */
    var BodyComponent = /** @class */ (function (_super) {
        __extends(BodyComponent, _super);
        /* CONSTRUCTOR */
        function BodyComponent(hostElement) {
            var _this = _super.call(this) || this;
            _this.hostElement = hostElement;
            /**
             * The fixtures that make up the body
             */
            _this.fixtures = [];
            return _this;
        }
        /* LIFECYCLE METHODS */
        BodyComponent.prototype.ngOnInit = function () {
            var _this = this;
            // Subscribe to Tick to animate (unless we're a static body)
            if (this.clock && this.body.m_type !== 0 /* B2_staticBody */) {
                this.subscribeTo(this.clock, function () { return _this.doTick(); });
            }
            // Build Fixtures List
            var fixture = this.body.GetFixtureList();
            while (fixture) {
                this.fixtures.push(fixture);
                fixture = fixture.GetNext();
            }
            // Set Fixture Positions
            this.doSetFixturePositions();
        };
        /* METHODS */
        /**
         * Process a tick of the clock
         */
        BodyComponent.prototype.doTick = function () {
            if (this.body && this.body.m_userData.tick) {
                this.body.GetUserData().tick();
            }
            this.doSetFixturePositions();
        };
        /**
         * Set the position of each fixture
         */
        BodyComponent.prototype.doSetFixturePositions = function () {
            var _this = this;
            if (this.body.IsAwake()) {
                this.fixtures.forEach(function (fixture) {
                    fixture.m_userData.path = SVGMathsUtility.fixturePath(_this.body, fixture);
                });
            }
        };
        __decorate([
            core.Input(),
            __metadata("design:type", thatBox2dLibrary.B2Body
            /**
             * The world to which this body belongs
             */
            )
        ], BodyComponent.prototype, "body", void 0);
        __decorate([
            core.Input(),
            __metadata("design:type", thatBox2dLibrary.B2World
            /**
             * The dynamic clock
             */
            )
        ], BodyComponent.prototype, "world", void 0);
        __decorate([
            core.Input(),
            __metadata("design:type", ngxComponentMixins.DynamicClock
            /* PROPERTIES */
            /**
             * The SVG path
             */
            )
        ], BodyComponent.prototype, "clock", void 0);
        __decorate([
            core.ViewChild('pathElement'),
            __metadata("design:type", core.ElementRef
            /**
             * The fixtures that make up the body
             */
            )
        ], BodyComponent.prototype, "pathElement", void 0);
        BodyComponent = __decorate([
            core.Component({
                selector: 'g[box2d]',
                template: '<svg:path ' +
                    '*ngFor="let fixture of fixtures" ' +
                    '[attr.d]="fixture.m_userData.path" ' +
                    '[attr.class]="fixture.m_userData.class"></svg:path>'
            }),
            __metadata("design:paramtypes", [core.ElementRef])
        ], BodyComponent);
        return BodyComponent;
    }(ngxComponentMixins.SubscriberMixin(ngxComponentMixins.BaseMixin)));

    var __extends$1 = (undefined && undefined.__extends) || (function () {
        var extendStatics = function (d, b) {
            extendStatics = Object.setPrototypeOf ||
                ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
                function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
            return extendStatics(d, b);
        };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    var __metadata$1 = (undefined && undefined.__metadata) || function (k, v) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
    };
    /**
     * A component for managing a Box2D world
     */
    var WorldComponent = /** @class */ (function (_super) {
        __extends$1(WorldComponent, _super);
        /* CONSTRUCTOR */
        function WorldComponent(hostElement) {
            var _this = _super.call(this) || this;
            _this.hostElement = hostElement;
            /* INPUTS */
            /**
             * How quickly does time tick
             * (Units are abitrary)
             */
            _this.speed = 5;
            /**
             * The number of velocity calculations to perform each tick
             */
            _this.velocityIterations = 6;
            /**
             * The number of position calculations to perform each tick
             */
            _this.positionIterations = 2;
            _this.hostElement.nativeElement.style.opacity = 0;
            _this.hostElement.nativeElement.style.transition = 'opacity 0.3s';
            return _this;
        }
        /* LIFECYCLE METHODS */
        WorldComponent.prototype.ngOnInit = function () {
            var _this = this;
            // Subscribe to dynamic clock
            this.subscribeTo(this.clock, function (_a) {
                var diff = _a.diff;
                return _this.doTick(diff);
            });
            // Fade in
            setTimeout(function () {
                _this.hostElement.nativeElement.style.opacity = 1;
            }, 500);
        };
        /* METHODS */
        /**
         * Process a tick of the clock
         */
        WorldComponent.prototype.doTick = function (diff) {
            if (diff) {
                this.world.Step(diff / ((11 - this.speed) * 40), this.velocityIterations, this.positionIterations);
            }
        };
        __decorate$1([
            core.Input(),
            __metadata$1("design:type", Number)
        ], WorldComponent.prototype, "speed", void 0);
        __decorate$1([
            core.Input(),
            __metadata$1("design:type", thatBox2dLibrary.B2World
            /**
             * The dynamic clock which governs the world time
             */
            )
        ], WorldComponent.prototype, "world", void 0);
        __decorate$1([
            core.Input(),
            __metadata$1("design:type", ngxComponentMixins.DynamicClock
            /**
             * The number of velocity calculations to perform each tick
             */
            )
        ], WorldComponent.prototype, "clock", void 0);
        __decorate$1([
            core.Input(),
            __metadata$1("design:type", Number)
        ], WorldComponent.prototype, "velocityIterations", void 0);
        __decorate$1([
            core.Input(),
            __metadata$1("design:type", Number)
        ], WorldComponent.prototype, "positionIterations", void 0);
        WorldComponent = __decorate$1([
            core.Component({
                selector: 'svg[box2d]',
                template: '<ng-content></ng-content>'
            }),
            __metadata$1("design:paramtypes", [core.ElementRef])
        ], WorldComponent);
        return WorldComponent;
    }(ngxComponentMixins.SubscriberMixin(ngxComponentMixins.BaseMixin)));

    var __decorate$2 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    /* MODULE DEFINITION */
    /**
     * A module of all the core angular components used within the application
     */
    var NgxBox2dModule = /** @class */ (function () {
        function NgxBox2dModule() {
        }
        NgxBox2dModule = __decorate$2([
            core.NgModule({
                imports: [common.CommonModule],
                declarations: [WorldComponent, BodyComponent],
                exports: [WorldComponent, BodyComponent]
            })
        ], NgxBox2dModule);
        return NgxBox2dModule;
    }());

    exports.BodyComponent = BodyComponent;
    exports.WorldComponent = WorldComponent;
    exports.SVGMathsUtility = SVGMathsUtility;
    exports.NgxBox2dModule = NgxBox2dModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
