var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, ViewChild, ElementRef } from '@angular/core';
import { BaseMixin, DynamicClock, SubscriberMixin } from 'ngx-component-mixins';
import { B2World, B2Body } from 'that-box2d-library';
import { SVGMathsUtility } from './svgmaths.utility';
/**
 * A component for attaching a Box2D body to a Angular Component
 */
var BodyComponent = /** @class */ (function (_super) {
    __extends(BodyComponent, _super);
    /* CONSTRUCTOR */
    function BodyComponent(hostElement) {
        var _this = _super.call(this) || this;
        _this.hostElement = hostElement;
        /**
         * The fixtures that make up the body
         */
        _this.fixtures = [];
        return _this;
    }
    /* LIFECYCLE METHODS */
    BodyComponent.prototype.ngOnInit = function () {
        var _this = this;
        // Subscribe to Tick to animate (unless we're a static body)
        if (this.clock && this.body.m_type !== 0 /* B2_staticBody */) {
            this.subscribeTo(this.clock, function () { return _this.doTick(); });
        }
        // Build Fixtures List
        var fixture = this.body.GetFixtureList();
        while (fixture) {
            this.fixtures.push(fixture);
            fixture = fixture.GetNext();
        }
        // Set Fixture Positions
        this.doSetFixturePositions();
    };
    /* METHODS */
    /**
     * Process a tick of the clock
     */
    BodyComponent.prototype.doTick = function () {
        if (this.body && this.body.m_userData.tick) {
            this.body.GetUserData().tick();
        }
        this.doSetFixturePositions();
    };
    /**
     * Set the position of each fixture
     */
    BodyComponent.prototype.doSetFixturePositions = function () {
        var _this = this;
        if (this.body.IsAwake()) {
            this.fixtures.forEach(function (fixture) {
                fixture.m_userData.path = SVGMathsUtility.fixturePath(_this.body, fixture);
            });
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", B2Body
        /**
         * The world to which this body belongs
         */
        )
    ], BodyComponent.prototype, "body", void 0);
    __decorate([
        Input(),
        __metadata("design:type", B2World
        /**
         * The dynamic clock
         */
        )
    ], BodyComponent.prototype, "world", void 0);
    __decorate([
        Input(),
        __metadata("design:type", DynamicClock
        /* PROPERTIES */
        /**
         * The SVG path
         */
        )
    ], BodyComponent.prototype, "clock", void 0);
    __decorate([
        ViewChild('pathElement'),
        __metadata("design:type", ElementRef
        /**
         * The fixtures that make up the body
         */
        )
    ], BodyComponent.prototype, "pathElement", void 0);
    BodyComponent = __decorate([
        Component({
            selector: 'g[box2d]',
            template: '<svg:path ' +
                '*ngFor="let fixture of fixtures" ' +
                '[attr.d]="fixture.m_userData.path" ' +
                '[attr.class]="fixture.m_userData.class"></svg:path>'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], BodyComponent);
    return BodyComponent;
}(SubscriberMixin(BaseMixin)));
export { BodyComponent };
//# sourceMappingURL=body.component.js.map